package qdos;

# qdos syscall package
#
##############################################################################
# qdos.pl provides the following interface to QDOS traps:
# 
# In the following examples, FH represents a FILEHANDLE.
# 
# &mt_prior ($job_id, $priority);	# set job priority
# 
# &mt_dmode ($mode, $display);		# set display mode
# 
# &io_pend (FH, $timeout);		# test pending I/O
# 
# &io_fline (FH, $timeout,$buf,$len);	# read line\n
#
# &io_fstrg (FH, $timeout,$buf,$len);	# read data
#
# &sd_bordr(FH, $timeout, $colour, $width);	# set window border
# 
# &sd_wdef(FH, $timeout, $colour, $width, @rect); # set window definition
# 						  # @rect = ($xp,$yp,$wid,$ht)
# 
# &sd_cure(FH, $timeout);		# enable cursor
# 
# &sd_curs(FH, $timeout);		# suppress cursor
# 
# &sd_pos(FH, $timeout, $xpos, $ypos);	# set cursor position
# 
# &sd_tab(FH, $timeout, $hpos);		# set horizontal position
# 
# &sd_nl(FH, $timeout);			# move to new line
# 
# &sd_pcol(FH, $timeout);		# move to previous column
# 
# &sd_ncol(FH, $timeout);		# move to new column
# 
# &sd_prow(FH, $timeout);		# move to previous row
# 
# &sd_nrow(FH, $timeout);		# move to new row
# 
# &sd_pixp(FH, $timeout, $xp, $yp);	# move to pixel position
# 
# &sd_scrol(FH, $timeout, $pix);	# scroll whole window
# 
# &sd_scrtp(FH, $timeout, $pix);	# scroll top of window
# 
# &sd_scrbt(FH, $timeout, $pix);	# scroll bottom of window
# 
# &sd_pan(FH, $timeout, $dist);		# pan all of window
# 
# &sd_panln(FH, $timeout, $dist);	# pan cursor line
# 
# &sd_panrt(FH, $timeout, $dist);	# pan right of cursor line
# 
# &sd_clear(FH, $timeout);		# clear whole screen
# 
# &sd_clrtp(FH, $timeout);		# clear top of screen
# 
# &sd_clrbt(FH, $timeout);		# clear bottom of screen
# 
# &sd_clrln(FH, $timeout);		# clear cursor line
# 
# &sd_clrrt(FH, $timeout);		# clear cursor to EOL
# 
# &sd_recol(FH, $timeout, @recols);	# transpose colours
# 
# &sd_setpa(FH, $timeout, $paper);	# set paper colour
# 
# &sd_setst(FH, $timeout, $strip);	# set strip colour
# 
# &sd_setin(FH, $timeout, $ink);	# set ink colour
# 
# &sd_setfl(FH, $timeout, $flag);	# set flash
# 
# &sd_setul(FH, $timeout, $flag);	# set underline
# 
# &sd_setmd(FH, $timeout, $mode);	# set write mode
# 
# &sd_setsz(FH, $timeout, $wid, $ht);	# set character height
# 
# &sd_fill(FH, $timeout, $colour, @block);# set fill (aka BLOCK)
# 
# &sd_donl(FH, $timeout);		# do pending new lines
# 
# &sd_point(FH, $timeout, @pos);	# plot a point
# 					# @pos = ($xp,$yp)
# 
# &sd_line(FH, $timeout, @pos);		# plot a line
# 					# @pos = ($x1,$y1,$x2,$y2)
# 
# &sd_arc(FH, $timeout, @pts);		# plot an arc 
# 					# @pts = ($x1,$y1,$x2,$y2,$angle)
# 
# &sd_elipse(FH, $timeout, @pts);	# plot an elipse/circle 
# 					# @pts = ($x,$y,$eccent,$radius,$angle)
# 
# &sd_scale(FH, $timeout, @pts);	# set graphics scale
# 					# @pts = ($scale,$ox,$oy)
# 
# &sd_gcur(FH, $timeout, @pts);		# set graphics cursor position
# 					# @pts = ($px,$py,$gx,$gy)
# 
# &sd_flood(FH, $timeout,$flag);	# set flood (aka FILL) mode
# 
###############################################################################
# Wrapper functions for the QDOS syscall().
# Replaces unreadable syscalls
#  syscall (3,0x2d,3,1,-1,fileno(STDOUT));
#  print "Hello\n";
#  syscall (3,0x2d,0,0,-1,fileno(STDOUT));
# with
#  &sd_setsz (STDOUT,-1, 3,1);
#  print "Hello\n";
#  &sd_setsz (STDOUT,-1, 0,0);
#
# Note:
#
# 1. Values cannot be returned. To implement this would involve an extension to
#    perl. This would be trivial to add as a module for perl 5, but I'm not
#    prepared to make massive changes to perl 4 to do this. perhaps one day
#    qdos will become sufficiently powerful to permit a perl 5 port.
#
# 2. All file handles are passed as file handles. This module converts them
#    to file numbers and the syscall implementation then gets the channel ID.
#
###############################################################################
#
# Author: Jonathan Hudson <rhudson@bigfoot.com>        07.11.98
#
###############################################################################
 
sub main'mt_prior
{
  local ($id,$pri) = @_;
  syscall (1,0x0b,$id,$pri);
}

sub main'mt_dmode
{
  local ($d1,$d2) = @_;
  syscall(1, 0x10, $d1, $d2);
}

sub main'mt_baud
{
  local ($baud) = @_;
  syscall (1,0x12,$baud);
}

sub main'io_pend
{
  local (*FH,$timeout) = @_;
  syscall (3,0,0,0,$timeout,fileno(FH));
}

sub main'io_fline
{
  local (*FH,$timeout,$buf, $len) = @_;
  $buf = "\0" x ($len+4);
  syscall(3,0x2,0,$len,$timeout,fileno(FH),$buf);
}

sub main'io_fstrg
{
  local (*FH,$timeout,$buf, $len) = @_;
  $buf = "\0" x ($len+4);
  syscall(3,0x3,0,$len,$timeout,fileno(FH),$buf);
}

sub main'sd_bordr
{
  local (*FH,$timeout,$col, $wid) = @_;
  syscall(3,0xc,$col,$wid,$timeout,fileno(FH));
}

sub main'sd_wdef
{
  local (*FH,$timeout,$col, $wid,@rect) = @_;
  syscall(3,0xd,$col,$wid,$timeout,fileno(FH),pack("nnnn",@rect));
}

sub main'sd_cure
{
  local (*FH,$timeout) = @_;
  syscall(3,0xe,0, 0, $timeout,fileno(FH));
}

sub main'sd_curs
{
  local (*FH,$timeout) = @_;
  syscall(3,0xf,0, 0, $timeout,fileno(FH));
}

sub main'sd_pos
{
  local (*FH,$timeout,$x,$y) = @_;
  syscall(3, 0x10, $x, $y, $timeout, fileno(FH));
}

sub main'sd_tab
{
  local (*FH,$timeout,$x) = @_;
  syscall(3, 0x11, $x, 0, $timeout, fileno(FH));
}

sub main'sd_nl
{
  local (*FH,$timeout,$y) = @_;
  syscall(3, 0x12, 0, $y, $timeout, fileno(FH));
}

sub main'sd_pcol
{
  local (*FH,$timeout) = @_;
  syscall(3, 0x13, 0, 0, $timeout, fileno(FH));
}

sub main'sd_ncol
{
  local (*FH,$timeout) = @_;
  syscall(3, 0x14, 0, 0, $timeout, fileno(FH));
}

sub main'sd_prow
{
  local (*FH,$timeout) = @_;
  syscall(3, 0x15, 0, 0, $timeout, fileno(FH));
}

sub main'sd_nrow
{
  local (*FH,$timeout) = @_;
  syscall(3, 0x16, 0, 0, $timeout, fileno(FH));
}

sub main'sd_pixp
{
  local (*FH,$timeout,$x,$y) = @_;
  syscall(3, 0x17, $x, $y, $timeout, fileno(FH));
}

sub main'sd_scrol
{
  local (*FH,$timeout,$pix) = @_;
  syscall(3, 0x18, $pix, 0, $timeout, fileno(FH));
}

sub main'sd_scrtp
{
  local (*FH,$timeout,$pix) = @_;
  syscall(3, 0x19, $pix, 0, $timeout, fileno(FH));
}

sub main'sd_scrbt
{
  local (*FH,$timeout,$pix) = @_;
  syscall(3, 0x1a, $pix, 0, $timeout, fileno(FH));
}

sub main'sd_pan
{
  local (*FH,$timeout,$dist) = @_;
  syscall(3, 0x1b, $dist, 0, $timeout, fileno(FH));
}

sub main'sd_panln
{
  local (*FH,$timeout,$dist) = @_;
  syscall(3, 0x1e, $dist, 0, $timeout, fileno(FH));
}
sub main'sd_panrt
{
  local (*FH,$timeout,$dist) = @_;
  syscall(3, 0x1f, $dist, 0, $timeout, fileno(FH));
}

sub main'sd_clear
{
  local (*FH,$timeout) = @_;
  syscall(3, 0x20, 0, 0, $timeout, fileno(FH));
}

sub main'sd_clrtp
{
  local (*FH,$timeout) = @_;
  syscall(3, 0x21, 0, 0, $timeout, fileno(FH));
}

sub main'sd_clrbt
{
  local (*FH,$timeout) = @_;
  syscall(3, 0x22, 0, 0, $timeout, fileno(FH));
}

sub main'sd_clrln
{
  local (*FH,$timeout) = @_;
  syscall(3, 0x23, 0, 0, $timeout, fileno(FH));
}

sub main'sd_clrrt
{
  local (*FH,$timeout) = @_;
  syscall(3, 0x24, 0, 0, $timeout, fileno(FH));
}

sub main'sd_recol
{
  local (*FH,$timeout,@recols) = @_;
  local ($recols);
  $recols = pack("cccccccc",@recols);
  syscall(3, 0x26, 0, 0, $timeout, fileno(FH), $recols);
}

sub main'sd_setpa
{
  local (*FH,$timeout,$col) = @_;
  syscall(3, 0x27, $col, 0, $timeout, fileno(FH));
}

sub main'sd_setst
{
  local (*FH,$timeout,$col) = @_;
  syscall(3, 0x28, $col, 0, $timeout, fileno(FH));
}

sub main'sd_setin
{
  local (*FH,$timeout,$col) = @_;
  syscall(3, 0x29, $col, 0, $timeout, fileno(FH));
}

sub main'sd_setfl
{
  local (*FH,$timeout,$flag) = @_;
  syscall(3, 0x2a, $flag, 0, $timeout, fileno(FH));
}

sub main'sd_setul
{
  local (*FH,$timeout,$flag) = @_;
  syscall(3, 0x2b, $flag, 0, $timeout, fileno(FH));
}

sub main'sd_setmd
{
  local (*FH,$timeout,$mode) = @_;
  syscall(3, 0x2c, $mode, 0, $timeout, fileno(FH));
}

sub main'sd_setsz
{
  local (*FH,$timeout,$wid,$hgt) = @_;
  syscall(3, 0x2d, $wid, $hgt, $timeout, fileno(FH));
}

sub main'sd_fill
{
  local (*FH,$timeout,$col,@block) = @_;
  syscall(3, 0x2e, $col, 0, $timeout, fileno(FH), pack("nnnn", @block));
}

sub main'sd_donl
{
  local (*FH,$timeout) = @_;
  syscall(3, 0x2f, $col, 0, $timeout, fileno(FH));
}

sub main'sd_point
{
  local (*FH,$timeout,@pts) = @_;
  syscall(3, 0x30, 0, 0, $timeout, fileno(FH), pack("dd", @pts));  
}

sub main'sd_line
{
  local (*FH,$timeout,@pts) = @_;
  syscall(3, 0x31, 0, 0, $timeout, fileno(FH), pack("dddd", @pts));  
}

sub main'sd_arc
{
  local (*FH,$timeout,@pts) = @_;
  syscall(3, 0x32, 0, 0, $timeout, fileno(FH), pack("ddddd", @pts));  
}

sub main'sd_elipse
{
  local (*FH,$timeout,@pts) = @_;
  syscall(3, 0x33, 0, 0, $timeout, fileno(FH), pack("ddddd", @pts));  
}

sub main'sd_scale
{
  local (*FH,$timeout,@pts) = @_;
  syscall(3, 0x34, 0, 0, $timeout, fileno(FH), pack("ddd", @pts));  
}

sub main'sd_gcur
{
  local (*FH,$timeout,@pts) = @_;
  syscall(3, 0x36, 0, 0, $timeout, fileno(FH), pack("dddd", @pts));  
}

sub main'sd_flood
{
  local (*FH,$timeout,$flag) = @_;
  syscall(3, 0x35, $flag, 0, $timeout, fileno(FH));  
}

1;


